/* Copyright (c) 2018 vesoft inc. All rights reserved.
 *
 * This source code is licensed under Apache 2.0 License,
 * attached with Common Clause Condition 1.0, found in the LICENSES directory.
 */
#include "base/Base.h"
#include "parser/MutateSentences.h"

namespace nebula {

std::string PropertyList::toString() const {
    std::string buf;
    buf.reserve(256);
    for (auto &prop : properties_) {
        buf += *prop;
        buf += ",";
    }
    if (!buf.empty()) {
        buf.resize(buf.size() - 1);
    }
    return buf;
}


std::string VertexTagItem::toString() const {
    std::string buf;
    buf.reserve(256);

    buf += *tagName_;
    if (properties_ != nullptr) {
        buf += "(";
        buf += properties_->toString();
        buf += ")";
    }

    return buf;
}


std::string VertexTagList::toString() const {
    std::string buf;
    buf.reserve(256);

    for (auto &item : tagItems_) {
        buf += item->toString();
        buf += ",";
    }
    if (!buf.empty()) {
        buf.resize(buf.size() - 1);
    }
    return buf;
}


std::string ValueList::toString() const {
    std::string buf;
    buf.reserve(256);
    for (auto &expr : values_) {
        buf += expr->toString();
        buf += ",";
    }
    if (!buf.empty()) {
        buf.resize(buf.size() - 1);
    }
    return buf;
}


std::string VertexRowItem::toString() const {
    std::string buf;
    buf.reserve(256);
    buf += std::to_string(id_);
    buf += ":";
    buf += "(";
    buf += values_->toString();
    buf += ")";
    return buf;
}


std::string VertexRowList::toString() const {
    std::string buf;
    buf.reserve(256);
    for (auto &item : rows_) {
        buf += item->toString();
        buf += ",";
    }
    if (!buf.empty()) {
        buf.resize(buf.size() - 1);
    }
    return buf;
}


std::string InsertVertexSentence::toString() const {
    std::string buf;
    buf.reserve(256);
    buf += "INSERT VERTEX ";
    buf += tagList_->toString();
    buf += " VALUES ";
    buf += rows_->toString();
    return buf;
}


std::string EdgeRowItem::toString() const {
    std::string buf;
    buf.reserve(256);

    buf += std::to_string(srcid_);
    buf += "->";
    buf += std::to_string(dstid_);
    if (rank_ != 0) {
        buf += "@";
        buf += std::to_string(rank_);
    }
    buf += ":";

    buf += "(";
    buf += values_->toString();
    buf += ")";
    return buf;
}


std::string EdgeRowList::toString() const {
    std::string buf;
    buf.reserve(256);
    for (auto &item : rows_) {
        buf += item->toString();
        buf += ",";
    }
    if (!buf.empty()) {
        buf.resize(buf.size() - 1);
    }
    return buf;
}


std::string InsertEdgeSentence::toString() const {
    std::string buf;
    buf.reserve(256);
    buf += "INSERT EDGE ";
    if (!overwritable_) {
        buf += "NO OVERWRITE ";
    }
    buf += *edge_;
    buf += "(";
    buf += properties_->toString();
    buf += ") VALUES";
    buf += rows_->toString();
    return buf;
}


std::string UpdateItem::toString() const {
    std::string buf;
    buf.reserve(256);
    buf += *field_;
    buf += "=";
    buf += value_->toString();
    return buf;
}


std::string UpdateList::toString() const {
    std::string buf;
    buf.reserve(256);
    for (auto &item : items_) {
        buf += item->toString();
        buf += ",";
    }
    if (!buf.empty()) {
        buf.resize(buf.size() - 1);
    }
    return buf;
}


std::string UpdateVertexSentence::toString() const {
    std::string buf;
    buf.reserve(256);
    buf += "UPDATE ";
    if (insertable_) {
        buf += "OR INSERT ";
    }
    buf += "VERTEX ";
    buf += std::to_string(vid_);
    buf += " SET ";
    buf += updateItems_->toString();
    if (whereClause_ != nullptr) {
        buf += " ";
        buf += whereClause_->toString();
    }
    if (yieldClause_ != nullptr) {
        buf += " ";
        buf += yieldClause_->toString();
    }

    return buf;
}


std::string UpdateEdgeSentence::toString() const {
    std::string buf;
    buf.reserve(256);
    buf += "UPDATE ";
    if (insertable_) {
        buf += "OR INSERT ";
    }
    buf += "EDGE ";
    buf += std::to_string(srcid_);
    buf += "->";
    buf += std::to_string(dstid_);
    buf += " SET ";
    buf += updateItems_->toString();
    if (whereClause_ != nullptr) {
        buf += " ";
        buf += whereClause_->toString();
    }
    if (yieldClause_ != nullptr) {
        buf += " ";
        buf += yieldClause_->toString();
    }

    return buf;
}

std::string DeleteVertexSentence::toString() const {
    std::string buf;
    buf.reserve(256);
    buf += "DELETE VERTEX ";
    buf += srcNodeList_->toString();
    if (whereClause_ != nullptr) {
        buf += " ";
        buf += whereClause_->toString();
    }
    return buf;
}

std::string EdgeList::toString() const {
    std::string buf;
    buf.reserve(256);
    for (auto edge : edges_) {
        buf += std::to_string(edge.first);
        buf += "->";
        buf += std::to_string(edge.second);
        buf += ",";
    }
    if (!buf.empty()) {
        buf.resize(buf.size() - 1);
    }
    return buf;
}

std::string DeleteEdgeSentence::toString() const {
    std::string buf;
    buf.reserve(256);
    buf += "DELETE EDGE ";
    buf += edgeList_->toString();
    if (whereClause_ != nullptr) {
        buf += " ";
        buf += whereClause_->toString();
    }
    return buf;
}

}   // namespace nebula
